//###########################################################################
// FILE:   main.c
// TITLE: VCU0 module Test in Concerto F28M35 processor
//     project derived from Blinky Example for F28M35x
//     and from 2837x_vcu0_rfft_256 for F2837xD.
//
//! AUTOR: Igor Kordunsky, 27-Nov 2014, Ver.3
//! \addtogroup control_example_list
//! <h1> VCU0_FFT (VCU0_FFT)</h1>
//!
//! This example calculates  16-bit FFT using VCU0 module.
//! then blinks LED2. If LED is blinking, it has calculated FFT without crash
//!
//  Deviation from 2837x_vcu0_rfft_256 example:
//  Test Input buffer "RIFFT16_256p_in_data"(which is placed into FLASH)
//  is copied into RAM "fft_input" before calculations,
//  because results are saved into same input buffer which is not possible when it is in FLASH
//  For speed:
//      assembler functions are placed in RAM,
//      Twiddle Factor table is copied into RAM.
// FOR PROPER WORK, BOTH fft_input[]  and fft_work_buffer[] MUST BE ALIGNED
// fft_input size MUST BE TWICE as fft_size because function cifft16_pack_asm
//  needs double input buffer to do packing
//  Results on M35 C28 coprocessor:
//----------------+---------------------------------|
//  Allowed Error |  within error   | outside error |
//----------------+-----------------+---------------|
//    EPSILON = 0 | pass = 329 bins | fail=183 bins |
//    EPSILON = 1 | pass = 511 bins | fail=  1 bin  |
//    EPSILON = 2 | pass = 512 bins | fail = 0 bins |
//----------------+-----------------+---------------+
//!
//###########################################################################
// $TI Release: F28M35x Support Library v204 $
// $Release Date: Fri Oct 17 14:52:33 CDT 2014 $
//###########################################################################
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// $
//###########################################################################
//*****************************************************************************
// includes
//*****************************************************************************

//#include "F28M35x_Device.h"     // Device Headerfile not needed for this project (and causes errors)
#include "DSP28x_Project.h"     // Device Headerfile and Examples Include File

#include <string.h>

#define F28_DATA_TYPES //prevents warnings about already defined types, like Int32
#include "vcu0_types.h"
#include "vcu0_fft.h"

//!
//! \addtogroup F28M35 VCU0_RIFFT_EXAMPLES Real Fast Fourier Transform (N = 256) Example

// @{

//*****************************************************************************
// defines
//*****************************************************************************
#define	 VCU0_FFT_STAGES	8
#define  VCU0_FFT_SIZE      (1<<VCU0_FFT_STAGES) // 256 Int16 words
#define  DATA_LENGTH        (VCU0_FFT_SIZE<<1)	 // 512 Int16 words
#define	 EPSILON		1 //allowed error vs precalculated test output
#define  RAM_TFtbl	//switch to put Twiddle Factors table in RAM
//*****************************************************************************
// globals
//*****************************************************************************
cfft16_t fft = rifft16_256P_DEFAULTS;

//there variables are automatically generated by linker, see "F28M35x_VCU0_C28_FLASH.cmd", SECTION .asm_text_ram
extern unsigned int asm_text_ram_loadstart;
extern unsigned int asm_text_ram_loadsize;
extern unsigned int asm_text_ram_runstart;

#ifdef __cplusplus
#pragma DATA_SECTION (".shadow");
#else
#pragma DATA_SECTION (fft_work_buffer, ".shadow");
#endif //__cplusplus
ComplexShort fft_work_buffer[VCU0_FFT_SIZE];
#pragma DATA_ALIGN ( fft_work_buffer, VCU0_FFT_SIZE<<1); // MUST BE ALIGNED

#ifdef __cplusplus
#pragma DATA_SECTION (".FFTinput");
#else
#pragma DATA_SECTION (fft_input, ".FFTinput");
#endif //__cplusplus
SINT16 fft_input[VCU0_FFT_SIZE<<1];
#pragma DATA_ALIGN ( fft_input, VCU0_FFT_SIZE<<1); //MUST BE ALIGNED

#ifdef __cplusplus
#pragma DATA_SECTION (".VCU_TwiddleFact");
#else
#pragma DATA_SECTION (fft_TF, ".VCU_TwiddleFact");
#endif //__cplusplus
SINT16 fft_TF[1024];

//Global Data
int pass ;//__attribute__ ((location(0x400))); location does not work
int fail;
//*****************************************************************************
// Function Prototypes
//*****************************************************************************
#pragma CODE_SECTION(rifft16, "ramfuncs");
void rifft16(cfft16_t *fft_hnd);
//*****************************************************************************
// function definitions
//*****************************************************************************
//!
//! \brief main routine for the 256-sample RFFT example
//! \return returns a 1
//!
//! This example shows how to use the vcu0 supported CFFT routines from the
//! library to perform a Real Inverse FFT. The input is placed in the .econst section
//! and needs to be aligned to the size of the input in words to allow the bit reverse
//! addressing in stage 1 of the FFT to work properly. The output, however, need
//! not be aligned to any boundary
//!
void main(void)
{
   unsigned long delay, k;
// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the F28M35x_SysCtrl.c file.
    InitSysCtrl();

// Step 2. Initialize GPIO:
// This example function is found in the F28M35x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
    InitGpio(); // Skipped for this example
    EALLOW;
    LED_0_DIR_REG = 1;
    EDIS;
    LED_0_DAT_REG = 1;// turn off LED
// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
    DINT;

#ifdef _FLASH
// Copy time critical code and Flash setup code to RAM
// This includes the following functions:  InitFlash();
// The  RamfuncsLoadStart, RamfuncsLoadSize, and RamfuncsRunStart
// symbols are created by the linker. Refer to the device .cmd file.
    memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
    InitFlash();
#endif

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the F28M35x_PieCtrl.c file.
    InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
    IER = 0x0000;
    IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in F28M35x_DefaultIsr.c.
// This function is found in F28M35x_PieVect.c.
    InitPieVectTable();

// Enable global Interrupts and higher priority real-time debug events:
    EINT;  // Enable Global interrupt INTM
    ERTM;  // Enable Global realtime interrupt DBGM
    //*************************************************************************
    // Running the FFT
    //*************************************************************************
    //! \b Running \b the \b FFT
    //! The user declares the cfft16_t Object with the appropriate
    //! init() and run() (often specified in the DEFAULT macros provided in the
    //! header file). The init() will load the twiddle factor table pointer
    //! to the structure. For the F28X7x devices, the twiddle factor tables are
    //! present in bootrom and may be used instead of the table provided with
    //! the library source code
    //! \code
    //*************************************************************************

	// NECESSARY Step if code placed into "ramfuncs": Copy ramfuncs into RAM (for speed)
#ifdef _FLASH
    memcpy(&asm_text_ram_runstart, &asm_text_ram_loadstart, (size_t)&asm_text_ram_loadsize);
#endif
	// Step 0: Copy input buffer into RAM
    memcpy(&fft_input, &RIFFT16_256p_in_data, (size_t)(VCU0_FFT_SIZE)); //copy 256 words into 512 buffer

	// Step 1: Initialize CFFT object
	  fft.ipcbptr = (int *)fft_input + 0; //input buffer pointer, MUST BE ALIGNED
	  fft.workptr = (int *)fft_work_buffer + 0; //work and ouptput buffer pointer, MUST BE ALIGNED
	  fft.init(&fft);
#ifdef  RAM_TFtbl
      memcpy(&fft_TF, &CFFT16_TF, sizeof(fft_TF)); //sizeof is 1024, and we need whole table
      fft.tfptr = fft_TF; //replace twddle factors pointer to RAM copy
#endif
	// Step 2: Pack the input; bit reverse it then run the FFT. Finally,
    // flip the real and imaginary parts of the output to obtain the
	// real inverse FFT
	  rifft16(&fft);

    //*************************************************************************
    //!
    //! \endcode
    //!
    //*************************************************************************
	// Step 3: Verify the result
	  pass=0;
	  fail=0;
	  for (k=0; k<DATA_LENGTH; k++)	  {
		  if (abs(RIFFT16_256p_out_data[k] - fft_input[k]) > EPSILON){
			  fail++;
		  }else{
			  pass++;
		  }
	  }

/*************************************************************
*  Results on M35 C28 coprocessor:
//----------------+---------------------------------|
//  Allowed Error |  within error   | outside error |
//----------------+-----------------+---------------|
//    EPSILON = 0 | pass = 329 bins | fail=183 bins |
//    EPSILON = 1 | pass = 511 bins | fail=  1 bin  |
//    EPSILON = 2 | pass = 512 bins | fail = 0 bins |
//----------------+-----------------+---------------+
//
// The failed bins have same sign of result, value differs
***************************************************************************************/
// Step 6. IDLE loop. IF PROGRAM POINTER IS HERE, IT DID NOT CRASH. Just sit and loop forever (optional):
    for(;;)
    {
    	LED_0_DAT_REG = 0;        // Turn on LED

    	// Delay for a bit.
        for(delay = 0; delay < 2000000; delay++)
        {
        }

        LED_0_DAT_REG = 1;        // Turn off LED

        // Delay for a bit.
        for(delay = 0; delay < 2000000; delay++)
        {
        }
    }
}

// End of main

//! \brief Real IFFT routine
//!
//! \param Handle to the structure, cfft16_t
//!
//! Calculates the Real Inverse FFT
//!
//! \return none
//

void rifft16(cfft16_t *fft_hnd)
{
  // pack complex IFFT input
  cifft16_pack_asm(fft_hnd);

  // complex FFT bit reversal
  cfft16_brev(fft_hnd);
  fft_hnd->calc(fft_hnd);

  //flip the real and imaginary and conjugate
  cfft16_flip_re_img_conj(fft_hnd);
}

// @} //addtogroup

// End of file




